@@ -21,54 +21,65 @@ NULL
 #' @return TRUE if an index file with ".tbi" or "bci" exists, and created later than VCF/BCF file
 #' @keywords internal
 hasIndex <- function(fileName) {
-    endsWith <- function(i, pattern = NULL) {
-        if (is.null(pattern)) {
-            return(FALSE)
-        }
-        if (length(fileName) != 1) {
-            stop("hasIndex() only take vector of length 1.")
-            return (FALSE)
-        }
-        p <- paste(pattern, "$", sep = "")
-        if (length(grep(x=i, pattern=p)) != 1) {
-            return (FALSE)
-        }
-        return (TRUE)
+  endsWith <- function(i, pattern = NULL) {
+    if (is.null(pattern)) {
+      return(FALSE)
     }
-    ## handle VCF
-    if (endsWith(fileName, ".vcf")) {
-        stop(gettextf("Input file is not bgzip compressed. Please use: [ bgzip %s ]", fileName))
-        return (FALSE)
+    if (length(fileName) != 1) {
+      stop("hasIndex() only take vector of length 1.")
+      return (FALSE)
     }
-    if (endsWith(fileName, ".vcf.gz")) {
-        fIndex <- paste(fileName, ".tbi", sep = "")
-        if (!file.exists(fIndex)) {
-            stop(gettextf("Cannot find index file (you can create it using: [ tabix -p vcf %s ])") , fileName)
-            return (FALSE)
-        }
-    }
-
-    ## handle BCF
-    if (endsWith(fileName, ".bcf")) {
-        stop(gettextf("Input file is not bgzip compressed. Please use: [ bgzip %s ]", fileName))
-        return (FALSE)
-    }
-    if (endsWith(fileName, ".bcf.gz")) {
-        fIndex <- paste(fileName, ".bci", sep = "")
-        if (!file.exists(fIndex)) {
-            stop(gettextf("Cannot find index file (you can create it using: [ bcftools index %s ])") , fileName)
-            return (FALSE)
-        }
-    }
-
-    ## Check index file timestamp
-    time.data <- file.info(fileName)$mtime
-    time.index <- file.info(fIndex)$mtime
-    if (time.data > time.index) {
-        stop(gettextf("Index file [ %s ]is older than data file [ %s ].", fIndex, fileName))
-        return(FALSE)
+    p <- paste(pattern, "$", sep = "")
+    if (length(grep(x=i, pattern=p)) != 1) {
+      return (FALSE)
     }
     return (TRUE)
+  }
+  ## handle VCF
+  fIndex <- NULL
+  if (endsWith(fileName, ".vcf")) {
+    stop(gettextf("Input file is not bgzip compressed. Please use: [ bgzip %s ]", fileName))
+    return (FALSE)
+  }
+  if (endsWith(fileName, ".vcf.gz")) {
+    fIndex <- paste(fileName, ".tbi", sep = "")
+    if (!file.exists(fIndex)) {
+      stop(gettextf("Cannot find index file (you can create it using: [ tabix -p vcf %s ])") , fileName)
+      return (FALSE)
+    }
+  }
+
+  ## handle BCF
+  if (endsWith(fileName, ".bcf")) {
+    stop(gettextf("Input file is not bgzip compressed. Please use: [ bgzip %s ]", fileName))
+    return (FALSE)
+  }
+  if (endsWith(fileName, ".bcf.gz")) {
+    fIndex <- paste(fileName, ".bci", sep = "")
+    if (!file.exists(fIndex)) {
+      stop(gettextf("Cannot find index file (you can create it using: [ bcftools index %s ])" , fileName))
+      return (FALSE)
+    }
+  }
+
+  ## Check index file
+  if (is.null(fIndex)) {
+    stop(gettextf("Cannot guess a valid tabix index, did your file have suffix: .vcf, .vcf.gz, .bcf, or .bcf.gz ?"))
+    return (FALSE)
+  }
+  if (!file.exists(fIndex)) {
+    stop(gettextf("The index file [ %s ] does not exists.", fIndex))
+    return (FALSE)    
+  }
+  
+  ## Check index file timestamp
+  time.data <- file.info(fileName)$mtime
+  time.index <- file.info(fIndex)$mtime
+  if (time.data > time.index) {
+    stop(gettextf("Index file [ %s ] is older than data file [ %s ].", fIndex, fileName))
+    return(FALSE)
+  }
+  return (TRUE)
 }
 
 #' Read a gene from VCF file and return a genotypes matrix
@@ -83,11 +94,11 @@ hasIndex <- function(fileName) {
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
 #' cfh <- readVCFToMatrixByRange(fileName, "1:196621007-196716634", "Nonsynonymous")
 readVCFToMatrixByRange <- function(fileName, range, annoType) {
-    stopifnot(file.exists(fileName), length(fileName) == 1, hasIndex(fileName))
-    storage.mode(fileName) <- "character"
-    storage.mode(range)    <- "character"
-    storage.mode(annoType) <- "character"
-    .Call("readVCFToMatrixByRange", fileName, range, annoType, PACKAGE="seqminer");
+  stopifnot(file.exists(fileName), length(fileName) == 1, hasIndex(fileName))
+  storage.mode(fileName) <- "character"
+  storage.mode(range)    <- "character"
+  storage.mode(annoType) <- "character"
+  .Call("readVCFToMatrixByRange", fileName, range, annoType, PACKAGE="seqminer");
 };
 
 #' Read a gene from VCF file and return a genotypes matrix
@@ -104,14 +115,14 @@ readVCFToMatrixByRange <- function(fileName, range, annoType) {
 #' geneFile = system.file("vcf/refFlat_hg19_6col.txt.gz", package = "seqminer")
 #' cfh <- readVCFToMatrixByGene(fileName, geneFile, "CFH", "Synonymous")
 readVCFToMatrixByGene <- function(fileName, geneFile, geneName, annoType) {
-    stopifnot(file.exists(fileName), length(fileName) == 1)
-    stopifnot(file.exists(geneFile), length(geneFile) == 1)
+  stopifnot(file.exists(fileName), length(fileName) == 1)
+  stopifnot(file.exists(geneFile), length(geneFile) == 1)
 
-    storage.mode(fileName) <- "character"
-    storage.mode(geneFile) <- "character"
-    storage.mode(geneName) <- "character"
-    storage.mode(annoType) <- "character"
-    .Call("readVCFToMatrixByGene", fileName, geneFile, geneName, annoType, PACKAGE="seqminer");
+  storage.mode(fileName) <- "character"
+  storage.mode(geneFile) <- "character"
+  storage.mode(geneName) <- "character"
+  storage.mode(annoType) <- "character"
+  .Call("readVCFToMatrixByGene", fileName, geneFile, geneName, annoType, PACKAGE="seqminer");
 };
 
 #' Read information from VCF file in a given range and return a list
@@ -130,14 +141,14 @@ readVCFToMatrixByGene <- function(fileName, geneFile, geneName, annoType) {
 #' cfh <- readVCFToListByRange(fileName, "1:196621007-196716634", "Nonsynonymous",
 #'                             c("CHROM", "POS"), c("AF", "AC"), c("GT") )
 readVCFToListByRange <- function(fileName, range, annoType, vcfColumn, vcfInfo, vcfIndv) {
-    stopifnot(file.exists(fileName), length(fileName) == 1, hasIndex(fileName))
-    storage.mode(fileName) <- "character"
-    storage.mode(range)    <- "character"
-    storage.mode(annoType) <- "character"
-    storage.mode(vcfColumn)<- "character"
-    storage.mode(vcfInfo)  <- "character"
-    storage.mode(vcfIndv)  <- "character"
-    .Call("readVCFToListByRange", fileName, range, annoType, vcfColumn, vcfInfo, vcfIndv, PACKAGE="seqminer");
+  stopifnot(file.exists(fileName), length(fileName) == 1, hasIndex(fileName))
+  storage.mode(fileName) <- "character"
+  storage.mode(range)    <- "character"
+  storage.mode(annoType) <- "character"
+  storage.mode(vcfColumn)<- "character"
+  storage.mode(vcfInfo)  <- "character"
+  storage.mode(vcfIndv)  <- "character"
+  .Call("readVCFToListByRange", fileName, range, annoType, vcfColumn, vcfInfo, vcfIndv, PACKAGE="seqminer");
 };
 
 #' Read information from VCF file in a given range and return a list
@@ -158,16 +169,16 @@ readVCFToListByRange <- function(fileName, range, annoType, vcfColumn, vcfInfo,
 #' cfh <- readVCFToListByGene(fileName, geneFile, "CFH", "Synonymous",
 #'                            c("CHROM", "POS"), c("AF", "AC"), c("GT") )
 readVCFToListByGene <- function(fileName, geneFile, geneName, annoType, vcfColumn, vcfInfo, vcfIndv) {
-    stopifnot(file.exists(fileName), length(fileName) == 1, hasIndex(fileName))
-    stopifnot(file.exists(geneFile), length(geneFile) == 1)
-    storage.mode(fileName) <- "character"
-    storage.mode(geneFile) <- "character"
-    storage.mode(geneName) <- "character"
-    storage.mode(annoType) <- "character"
-    storage.mode(vcfColumn)<- "character"
-    storage.mode(vcfInfo)  <- "character"
-    storage.mode(vcfIndv)  <- "character"
-    .Call("readVCFToListByGene", fileName, geneFile, geneName, annoType, vcfColumn, vcfInfo, vcfIndv, PACKAGE="seqminer");
+  stopifnot(file.exists(fileName), length(fileName) == 1, hasIndex(fileName))
+  stopifnot(file.exists(geneFile), length(geneFile) == 1)
+  storage.mode(fileName) <- "character"
+  storage.mode(geneFile) <- "character"
+  storage.mode(geneName) <- "character"
+  storage.mode(annoType) <- "character"
+  storage.mode(vcfColumn)<- "character"
+  storage.mode(vcfInfo)  <- "character"
+  storage.mode(vcfIndv)  <- "character"
+  .Call("readVCFToListByGene", fileName, geneFile, geneName, annoType, vcfColumn, vcfInfo, vcfIndv, PACKAGE="seqminer");
 };
 
 #' Read association statistics by gene from METAL-format files. Both score statistics and covariance statistics will be extracted.
@@ -185,18 +196,18 @@ readVCFToListByGene <- function(fileName, geneFile, geneName, annoType, vcfColum
 #' geneFile = system.file("vcf/refFlat_hg19_6col.txt.gz", package = "seqminer")
 #' cfh <- rvmeta.readDataByGene(scoreFileName, covFileName, geneFile, "CFH")
 rvmeta.readDataByGene <- function(scoreTestFiles, covFiles, geneFile, geneName) {
-    stopifnot(file.exists(scoreTestFiles))
-    stopifnot(is.null(covFiles) || (file.exists(covFiles) && length(covFiles) == length(scoreTestFiles)))
-    stopifnot(file.exists(geneFile), length(geneFile) == 1)
-    storage.mode(scoreTestFiles) <- "character"
-    storage.mode(covFiles) <- "character"
-    storage.mode(geneFile) <- "character"
-    storage.mode(geneName) <- "character"
-    if (is.null(covFiles)) {
-        .Call("rvMetaReadDataByGene", scoreTestFiles, "", geneFile, geneName, PACKAGE="seqminer");
-    } else {
-        .Call("rvMetaReadDataByGene", scoreTestFiles, covFiles, geneFile, geneName, PACKAGE="seqminer");
-    }
+  stopifnot(file.exists(scoreTestFiles))
+  stopifnot(is.null(covFiles) || (file.exists(covFiles) && length(covFiles) == length(scoreTestFiles)))
+  stopifnot(file.exists(geneFile), length(geneFile) == 1)
+  storage.mode(scoreTestFiles) <- "character"
+  storage.mode(covFiles) <- "character"
+  storage.mode(geneFile) <- "character"
+  storage.mode(geneName) <- "character"
+  if (is.null(covFiles)) {
+    .Call("rvMetaReadDataByGene", scoreTestFiles, "", geneFile, geneName, PACKAGE="seqminer");
+  } else {
+    .Call("rvMetaReadDataByGene", scoreTestFiles, covFiles, geneFile, geneName, PACKAGE="seqminer");
+  }
 };
 
 #' Read association statistics by range from METAL-format files. Both score statistics and covariance statistics will be extracted.
@@ -213,18 +224,18 @@ rvmeta.readDataByGene <- function(scoreTestFiles, covFiles, geneFile, geneName)
 #' geneFile = system.file("vcf/refFlat_hg19_6col.txt.gz", package = "seqminer")
 #' cfh <- rvmeta.readDataByRange(scoreFileName, covFileName, "1:196621007-196716634")
 rvmeta.readDataByRange <- function (scoreTestFiles, covFiles, ranges) {
-    stopifnot(file.exists(scoreTestFiles))
-    stopifnot(is.null(covFiles) || (file.exists(covFiles) && length(covFiles) == length(scoreTestFiles)))
-    storage.mode(scoreTestFiles) <- "character"
-    storage.mode(covFiles) <- "character"
-    storage.mode(ranges) <- "character"
-    if (is.null(covFiles)) {
-        .Call("rvMetaReadDataByRange", scoreTestFiles, "",
-              ranges, PACKAGE = "seqminer")
-    } else {
-        .Call("rvMetaReadDataByRange", scoreTestFiles, covFiles,
-              ranges, PACKAGE = "seqminer")
-    }
+  stopifnot(file.exists(scoreTestFiles))
+  stopifnot(is.null(covFiles) || (file.exists(covFiles) && length(covFiles) == length(scoreTestFiles)))
+  storage.mode(scoreTestFiles) <- "character"
+  storage.mode(covFiles) <- "character"
+  storage.mode(ranges) <- "character"
+  if (is.null(covFiles)) {
+    .Call("rvMetaReadDataByRange", scoreTestFiles, "",
+          ranges, PACKAGE = "seqminer")
+  } else {
+    .Call("rvMetaReadDataByRange", scoreTestFiles, covFiles,
+          ranges, PACKAGE = "seqminer")
+  }
 }
 
 #' Read covariance by range from METAL-format files.
@@ -238,10 +249,10 @@ rvmeta.readDataByRange <- function (scoreTestFiles, covFiles, ranges) {
 #' covFileName = system.file("rvtests/rvtest.MetaCov.assoc.gz", package = "seqminer")
 #' cfh <- rvmeta.readCovByRange(covFileName, "1:196621007-196716634")
 rvmeta.readCovByRange <- function(covFile, tabixRange) {
-    stopifnot(file.exists(covFile))
-    storage.mode(covFile) <- "character"
-    storage.mode(tabixRange) <- "character"
-    .Call("readCovByRange", covFile, tabixRange, PACKAGE="seqminer");
+  stopifnot(file.exists(covFile))
+  storage.mode(covFile) <- "character"
+  storage.mode(tabixRange) <- "character"
+  .Call("readCovByRange", covFile, tabixRange, PACKAGE="seqminer");
 };
 
 #' Read score test statistics by range from METAL-format files.
@@ -255,10 +266,10 @@ rvmeta.readCovByRange <- function(covFile, tabixRange) {
 #' scoreFileName = system.file("rvtests/rvtest.MetaScore.assoc.anno.gz", package = "seqminer")
 #' cfh <- rvmeta.readScoreByRange(scoreFileName, "1:196621007-196716634")
 rvmeta.readScoreByRange <- function(scoreTestFiles, tabixRange) {
-    stopifnot(file.exists(scoreTestFiles))
-    storage.mode(scoreTestFiles) <- "character"
-    storage.mode(tabixRange) <- "character"
-    .Call("readScoreByRange", scoreTestFiles, tabixRange, PACKAGE="seqminer");
+  stopifnot(file.exists(scoreTestFiles))
+  storage.mode(scoreTestFiles) <- "character"
+  storage.mode(tabixRange) <- "character"
+  .Call("readScoreByRange", scoreTestFiles, tabixRange, PACKAGE="seqminer");
 };
 
 #' Read skew by range from METAL-format files.
@@ -272,10 +283,10 @@ rvmeta.readScoreByRange <- function(scoreTestFiles, tabixRange) {
 #' skewFileName = system.file("rvtests/rvtest.MetaSkew.assoc.gz", package = "seqminer")
 #' cfh <- rvmeta.readSkewByRange(skewFileName, "1:196621007-196716634")
 rvmeta.readSkewByRange <- function(skewFile, tabixRange) {
-    stopifnot(file.exists(skewFile))
-    storage.mode(skewFile) <- "character"
-    storage.mode(tabixRange) <- "character"
-    .Call("readSkewByRange", skewFile, tabixRange, PACKAGE="seqminer");
+  stopifnot(file.exists(skewFile))
+  storage.mode(skewFile) <- "character"
+  storage.mode(tabixRange) <- "character"
+  .Call("readSkewByRange", skewFile, tabixRange, PACKAGE="seqminer");
 };
 
 #' Read tabix file, similar to running tabix in command line.
@@ -289,10 +300,10 @@ rvmeta.readSkewByRange <- function(skewFile, tabixRange) {
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
 #' snp <- tabix.read(fileName, "1:196623337-196632470")
 tabix.read <- function(tabixFile, tabixRange) {
-    stopifnot(file.exists(tabixFile))
-    storage.mode(tabixFile) <- "character"
-    storage.mode(tabixRange) <- "character"
-    .Call("readTabixByRange", tabixFile, tabixRange, PACKAGE="seqminer");
+  stopifnot(file.exists(tabixFile))
+  storage.mode(tabixFile) <- "character"
+  storage.mode(tabixRange) <- "character"
+  .Call("readTabixByRange", tabixFile, tabixRange, PACKAGE="seqminer");
 }
 
 #' Read tabix file, similar to running tabix in command line.
@@ -306,16 +317,16 @@ tabix.read <- function(tabixFile, tabixRange) {
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
 #' snp <- tabix.read.header(fileName)
 tabix.read.header <- function(tabixFile, skippedLine = FALSE) {
-    stopifnot(file.exists(tabixFile))
-    storage.mode(tabixFile) <- "character"
-    header <- .Call("readTabixHeader", tabixFile, PACKAGE="seqminer");
-    ret <- list(header = header)
-    if (skippedLine) {
-        skipped <- .Call("readTabixSkippedLine", tabixFile, PACKAGE="seqminer");
-        ret[[length(ret) + 1]] <- skipped
-        names(ret)[2] <- "skippedLine"
-    }
-    ret
+  stopifnot(file.exists(tabixFile))
+  storage.mode(tabixFile) <- "character"
+  header <- .Call("readTabixHeader", tabixFile, PACKAGE="seqminer");
+  ret <- list(header = header)
+  if (skippedLine) {
+    skipped <- .Call("readTabixSkippedLine", tabixFile, PACKAGE="seqminer");
+    ret[[length(ret) + 1]] <- skipped
+    names(ret)[2] <- "skippedLine"
+  }
+  ret
 }
 
 #' Read tabix file, similar to running tabix in command line.
@@ -331,59 +342,59 @@ tabix.read.header <- function(tabixFile, skippedLine = FALSE) {
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
 #' snp <- tabix.read.table(fileName, "1:196623337-196632470")
 tabix.read.table <- function(tabixFile, tabixRange, col.names = TRUE, stringsAsFactors = FALSE) {
-    stopifnot(file.exists(tabixFile))
-    storage.mode(tabixFile) <- "character"
-    storage.mode(tabixRange) <- "character"
-    header <- .Call("readTabixHeader", tabixFile, PACKAGE = "seqminer")
-    body <- .Call("readTabixByRange", tabixFile, tabixRange, PACKAGE="seqminer");
+  stopifnot(file.exists(tabixFile))
+  storage.mode(tabixFile) <- "character"
+  storage.mode(tabixRange) <- "character"
+  header <- .Call("readTabixHeader", tabixFile, PACKAGE = "seqminer")
+  body <- .Call("readTabixByRange", tabixFile, tabixRange, PACKAGE="seqminer");
 
-    ## parse body to a table
-    body <- do.call(rbind, str_split(body, "\t"))
-    body <- as.data.frame(body, stringsAsFactors = FALSE)
-    if (ncol(body) > 0) {
-        for (i in 1:ncol(body)) {
-            body[,i] <- type.convert(body[,i], as.is = !stringsAsFactors)
-        }
-
-        num.col <- ncol(body)
-        header <- header[nchar(header) > 0]
-        if (length(header) == 0 || !col.names) {
-            colNames <- paste0("V", 1L:num.col)
-        } else {
-            hdrLine <- header[length(header)]
-            hdrLine <- str_replace(hdrLine, "^#", "")
-            colNames <- make.names(str_split(hdrLine, "\t")[[1]])
-            if (length(colNames) > ncol(body)) {
-                colNames <- colNames[1:ncol(body)]
-            } else if (length(colNames) < ncol(body)) {
-                tmpNames <- paste0("V", 1L:num.col)
-                tmpNames[1:length(colNames )] <- colNames
-                colNames <- tmpNames
-            }
-        }
-        colnames(body) <- colNames
+  ## parse body to a table
+  body <- do.call(rbind, str_split(body, "\t"))
+  body <- as.data.frame(body, stringsAsFactors = FALSE)
+  if (ncol(body) > 0) {
+    for (i in 1:ncol(body)) {
+      body[,i] <- type.convert(body[,i], as.is = !stringsAsFactors)
     }
 
-    body
+    num.col <- ncol(body)
+    header <- header[nchar(header) > 0]
+    if (length(header) == 0 || !col.names) {
+      colNames <- paste0("V", 1L:num.col)
+    } else {
+      hdrLine <- header[length(header)]
+      hdrLine <- str_replace(hdrLine, "^#", "")
+      colNames <- make.names(str_split(hdrLine, "\t")[[1]])
+      if (length(colNames) > ncol(body)) {
+        colNames <- colNames[1:ncol(body)]
+      } else if (length(colNames) < ncol(body)) {
+        tmpNames <- paste0("V", 1L:num.col)
+        tmpNames[1:length(colNames )] <- colNames
+        colNames <- tmpNames
+      }
+    }
+    colnames(body) <- colNames
+  }
+
+  body
 }
 
 .onAttach <- function(libname, pkgname){
-    newVersionLink = "http://zhanxw.com:8080/seqminer/version"
-    conn <- url(newVersionLink)
-    ret <- tryCatch(readLines(conn, n = 2), error = function(e) {NULL})
-    close(conn)
+  newVersionLink = "http://zhanxw.com:8080/seqminer/version"
+  conn <- url(newVersionLink)
+  ret <- tryCatch(readLines(conn, n = 2), error = function(e) {NULL})
+  close(conn)
 
-    if (!is.null(ret) && length(ret) == 2) {
-        version <- ret[1]
-        if (utils::packageVersion("seqminer") < version) {
-            if (length(ret) > 1) {
-                packageStartupMessage(ret[2])
-            } else {
-                packageStartupMessage("Found new version of seqminer: ", ret)
-            }
-        }
+  if (!is.null(ret) && length(ret) == 2) {
+    version <- ret[1]
+    if (utils::packageVersion("seqminer") < version) {
+      if (length(ret) > 1) {
+        packageStartupMessage(ret[2])
+      } else {
+        packageStartupMessage("Found new version of seqminer: ", ret)
+      }
     }
-    ## packageStartupMessage("seqminer loaded ...")
+  }
+  ## packageStartupMessage("seqminer loaded ...")
 }
 
 #' Read null model statistics
@@ -396,50 +407,50 @@ tabix.read.table <- function(tabixFile, tabixRange, col.names = TRUE, stringsAsF
 #' @examples
 #' scoreFileName = system.file("rvtests/rvtest.MetaScore.assoc.anno.gz", package = "seqminer")
 rvmeta.readNullModel <- function(scoreTestFiles) {
-    stopifnot(file.exists(scoreTestFiles))
-    read.null.model <- function(fn) {
-        ## cat("gzFile, fn = ", fn, "\n")
-        f <- gzfile(fn, "rb")
-        ret = list()
-        beginStore = FALSE
-        while(TRUE) {
-            suppressWarnings(p <- readLines(con = f, n = 1, warn = FALSE))
-            if (substr(p, 1, 1) != "#") {
-                break
-            }
-            #print(p)
-            if (!beginStore) {
-                if ( p[1] == "##NullModelEstimates") {
-                    beginStore = TRUE
-                    next
-                }
-            } else {
-                ret[length(ret) + 1] <- p
-            }
+  stopifnot(file.exists(scoreTestFiles))
+  read.null.model <- function(fn) {
+    ## cat("gzFile, fn = ", fn, "\n")
+    f <- gzfile(fn, "rb")
+    ret = list()
+    beginStore = FALSE
+    while(TRUE) {
+      suppressWarnings(p <- readLines(con = f, n = 1, warn = FALSE))
+      if (substr(p, 1, 1) != "#") {
+        break
+      }
+      #print(p)
+      if (!beginStore) {
+        if ( p[1] == "##NullModelEstimates") {
+          beginStore = TRUE
+          next
         }
-        close(f)
-        ret
+      } else {
+        ret[length(ret) + 1] <- p
+      }
     }
-    model.to.matrix <- function(ret) {
-        if (is.null(ret) || length(ret) < 1) {
-            return(numeric(0))
-        }
-        ## library(stringr)
-        ## ret <- read.null.model(ret)
-        ret <- lapply(ret, function(x) { str_split(str_replace(x, "## - ", ""), "\t")[[1]]})
-        cnames <- ret[[1]]
-        ret[[1]] <- NULL
-        rnames <- lapply(ret, function(x) {x[1]})
-        rnames <- do.call(c, rnames)
-        data <- lapply(ret, function(x) {x[-1]})
-        mat <- do.call(rbind, data)
-        colnames(mat) <- cnames[-1]
-        rownames(mat) <- rnames
-        suppressWarnings(class(mat) <- "numeric")
-        mat
-    }
-    ret <- lapply(scoreTestFiles, function(x) { model.to.matrix(read.null.model(x))})
+    close(f)
     ret
+  }
+  model.to.matrix <- function(ret) {
+    if (is.null(ret) || length(ret) < 1) {
+      return(numeric(0))
+    }
+    ## library(stringr)
+    ## ret <- read.null.model(ret)
+    ret <- lapply(ret, function(x) { str_split(str_replace(x, "## - ", ""), "\t")[[1]]})
+    cnames <- ret[[1]]
+    ret[[1]] <- NULL
+    rnames <- lapply(ret, function(x) {x[1]})
+    rnames <- do.call(c, rnames)
+    data <- lapply(ret, function(x) {x[-1]})
+    mat <- do.call(rbind, data)
+    colnames(mat) <- cnames[-1]
+    rownames(mat) <- rnames
+    suppressWarnings(class(mat) <- "numeric")
+    mat
+  }
+  ret <- lapply(scoreTestFiles, function(x) { model.to.matrix(read.null.model(x))})
+  ret
 };
 
 #' Write score-based association statistics files.
@@ -457,11 +468,11 @@ rvmeta.readNullModel <- function(scoreTestFiles) {
 #' cfh <- rvmeta.readDataByRange(scoreFileName, covFileName, "1:196621007-196716634")
 #' rvmeta.writeScoreData(cfh, "cfh.MetaScore.assoc")
 rvmeta.writeScoreData <- function (rvmetaData, outName, createIndex = FALSE) {
-    storage.mode(outName) <- "character"
-    .Call("rvMetaWriteScoreData", rvmetaData, outName, PACKAGE="seqminer");
-    if (createIndex) {
-        tabix.createIndex.meta(outName)
-    }
+  storage.mode(outName) <- "character"
+  .Call("rvMetaWriteScoreData", rvmetaData, outName, PACKAGE="seqminer");
+  if (createIndex) {
+    tabix.createIndex.meta(outName)
+  }
 }
 
 #' Write covariance association statistics files.
@@ -478,9 +489,9 @@ rvmeta.writeScoreData <- function (rvmetaData, outName, createIndex = FALSE) {
 #' cfh <- rvmeta.readDataByRange(scoreFileName, covFileName, "1:196621007-196716634")
 #' rvmeta.writeCovData(cfh, "cfh.MetaCov.assoc")
 rvmeta.writeCovData <- function (rvmetaData, outName) {
-    storage.mode(outName) <- "character"
-    .Call("rvMetaWriteCovData", rvmetaData, outName, PACKAGE="seqminer");
-    tabix.createIndex.meta(outName)
+  storage.mode(outName) <- "character"
+  .Call("rvMetaWriteCovData", rvmetaData, outName, PACKAGE="seqminer");
+  tabix.createIndex.meta(outName)
 }
 
 #' Create tabix index file, similar to running tabix in command line.
@@ -498,14 +509,14 @@ rvmeta.writeCovData <- function (rvmetaData, outName) {
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
 #' tabix.createIndex(fileName, 1, 2, 0, '#', 0)
 tabix.createIndex <- function(bgzipFile, sequenceColumn = 1, startColumn = 4, endColumn = 5, metaChar = "#", skipLines = 0) {
-    stopifnot(file.exists(bgzipFile))
-    storage.mode(bgzipFile) <- "character"
-    storage.mode(sequenceColumn) <- "integer"
-    storage.mode(startColumn) <- "integer"
-    storage.mode(endColumn) <- "integer"
-    storage.mode(metaChar) <- "character"
-    storage.mode(skipLines) <- "integer"
-    .Call("createTabixIndex", bgzipFile, sequenceColumn, startColumn, endColumn, metaChar, skipLines, PACKAGE="seqminer");
+  stopifnot(file.exists(bgzipFile))
+  storage.mode(bgzipFile) <- "character"
+  storage.mode(sequenceColumn) <- "integer"
+  storage.mode(startColumn) <- "integer"
+  storage.mode(endColumn) <- "integer"
+  storage.mode(metaChar) <- "character"
+  storage.mode(skipLines) <- "integer"
+  .Call("createTabixIndex", bgzipFile, sequenceColumn, startColumn, endColumn, metaChar, skipLines, PACKAGE="seqminer");
 }
 
 #' Create tabix index for bgzipped VCF file
@@ -518,8 +529,8 @@ tabix.createIndex <- function(bgzipFile, sequenceColumn = 1, startColumn = 4, en
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
 #' tabix.createIndex.vcf(fileName)
 tabix.createIndex.vcf <- function(bgzipVcfFile) {
-    stopifnot(file.exists(bgzipVcfFile))
-    tabix.createIndex(bgzipVcfFile, 1, 2, 0, '#', 0)
+  stopifnot(file.exists(bgzipVcfFile))
+  tabix.createIndex(bgzipVcfFile, 1, 2, 0, '#', 0)
 }
 
 #' Create tabix index for bgzipped MetaScore/MetaCov file
@@ -534,14 +545,26 @@ tabix.createIndex.vcf <- function(bgzipVcfFile) {
 #' fileName = system.file("rvtests/rvtest.MetaScore.assoc.anno.gz", package = "seqminer")
 #' tabix.createIndex.meta(fileName)
 tabix.createIndex.meta <- function(bgzipFile) {
-    stopifnot(file.exists(bgzipFile))
-    tabix.createIndex(bgzipFile, 1, 2, 2, '#', 0)
+  stopifnot(file.exists(bgzipFile))
+  tabix.createIndex(bgzipFile, 1, 2, 2, '#', 0)
 }
 
 ##################################################
 ## Annotations --------------------
 ##################################################
 
+#' validate the @param inVcf can be opened, and @param outtVcf can be write to
+#' will stop if errors occur
+#' @param inVcf: input file
+#' @param outVcf: output file
+#' @return NULL
+verifyFilename <- function(inVcf, outVcf) {
+  if (!file.exists(inVcf)) {
+    stop("Cannot open input file")
+  }
+  tryCatch(file.create(outVcf), warning = function(w) {stop("Cannot create output file")})
+}
+
 #' Validate annotate parameter is valid
 #' @param param a list of annotation elements
 #' @param debug show extra debug information or not
@@ -549,48 +572,103 @@ tabix.createIndex.meta <- function(bgzipFile) {
 #                second element is a message string
 #' @export
 validateAnnotationParameter <- function(param, debug = FALSE) {
-    status <- TRUE
-    msg <- vector("character")
-    if (debug) {
-        print(param)
-    }
-    if (is.null(param$reference) || !file.exists(param$reference)) {
-        msg <- c(msg, "Reference file does not exist")
-    }
-    if (is.null(param$geneFile) || !file.exists(param$geneFile)) {
+  status <- TRUE
+  msg <- vector("character")
+  if (debug) {
+    print(param)
+  }
+  if (is.null(param$reference) || !file.exists(param$reference)) {
+    msg <- c(msg, "Reference file does not exist")
+  }
+  if (is.null(param$geneFile) || !file.exists(param$geneFile)) {
+    status <- FALSE
+    msg <- c(msg, "Gene file does not exist")
+  }
+  if (param$geneFileFormat != "refFlat") {
+    status <- FALSE
+    msg <- c(msg, "Gene file format does not exist")
+  }
+  if (is.null(param$codonFile) || !file.exists(param$codonFile)) {
+    status <- FALSE
+    msg <- c(msg, "Codon file does not exist")
+  }
+  if (is.null(param$priorityFile) || !file.exists(param$priorityFile)) {
+    status <- FALSE
+    msg <- c(msg, "Priority file does not exist")
+  }
+  if (param$upstreamRange <= 0) {
+    status <- FALSE
+    msg <- c(msg, "Upstream range is less than zero")
+  }
+  if (param$downstreamRange <= 0) {
+    status <- FALSE
+    msg <- c(msg, "Downstream range is less than zero")
+  }
+  if (param$spliceIntoExon <= 0) {
+    status <- FALSE
+    msg <- c(msg, "SpliceIntoExon range is less than zero")
+  }
+  if (param$spliceIntoIntron <= 0) {
+    status <- FALSE
+    msg <- c(msg, "SpliceIntoIntron range is less than zero")
+  }
+  if (!is.null(param$bed)) {
+    ##library(stringr)
+    opt <- str_split(param$bed, ",")
+    n <- length(opt)
+    parsed <- lapply(opt, function(x){unlist(str_split(x, "="))})
+    for (i in seq_along(n)) {
+      if (length(parsed[[i]]) != 2) {
         status <- FALSE
-        msg <- c(msg, "Gene file does not exist")
-    }
-    if (param$geneFileFormat != "refFlat") {
+        msg <- c(msg, paste("Cannot understand bed option: ", parsed[[i]]))
+        next
+      }
+      if (!file.exists(parsed[[i]][2])) {
         status <- FALSE
-        msg <- c(msg, "Gene file format does not exist")
+        msg <- c(msg, paste("BED resource file not exists: ", parsed[[i]][2]))
+        next        
+      }
     }
-    if (is.null(param$codonFile) || !file.exists(param$codonFile)) {
+  }
+  if (!is.null(param$genomeScore)) {
+    ##library(stringr)
+    opt <- str_split(param$genomeScore, ",")
+    n <- length(opt)
+    parsed <- lapply(opt, function(x){unlist(str_split(x, "="))})
+    for (i in seq_along(n)) {
+      if (length(parsed[[i]]) != 2) {
         status <- FALSE
-        msg <- c(msg, "Codon file does not exist")
-    }
-    if (is.null(param$priorityFile) || !file.exists(param$priorityFile)) {
+        msg <- c(msg, paste("Cannot understand genomeScore option: ", parsed[[i]]))
+        next
+      }
+      if (!file.exists(parsed[[i]][2])) {
         status <- FALSE
-        msg <- c(msg, "Priority file does not exist")
+        msg <- c(msg, paste("Genomescore resource file not exists: ", parsed[[i]][2]))
+        next        
+      }
     }
-    if (param$upstreamRange <= 0) {
+  }
+  if (!is.null(param$tabix)) {
+    ##library(stringr)
+    opt <- str_split(param$tabix, "),")
+    n <- length(opt)
+    parsed <- lapply(opt, function(x){unlist(str_split(x, "\\("))})
+    for (i in seq_along(n)) {
+      if (length(parsed[[i]]) != 2) {
         status <- FALSE
-        msg <- c(msg, "Upstream range is less than zero")
-    }
-    if (param$downstreamRange <= 0) {
+        msg <- c(msg, paste("Cannot understand tabix option: ", parsed[[i]]))
+        next
+      }
+      if (!file.exists(parsed[[i]][1])) {
         status <- FALSE
-        msg <- c(msg, "Downstream range is less than zero")
+        msg <- c(msg, paste("Tabix resource file not exists: ", parsed[[i]][1]))
+        next        
+      }
     }
-    if (param$spliceIntoExon <= 0) {
-        status <- FALSE
-        msg <- c(msg, "SpliceIntoExon range is less than zero")
-    }
-    if (param$spliceIntoIntron <= 0) {
-        status <- FALSE
-        msg <- c(msg, "SpliceIntoIntron range is less than zero")
-    }
-    res <- list(status, paste(msg, sep = " \n"))
-    return(res)
+  }
+  
+  res <- list(status, paste(msg, sep = " \n"))
+  return(res)
 }
 
 #' Construct a usable set of annotation parameters
@@ -598,24 +676,30 @@ validateAnnotationParameter <- function(param, debug = FALSE) {
 #' @return list, a complete list of supported parameters
 #' @export
 makeAnnotationParameter <- function(param = NULL) {
-    ## default parameters
-    defaultParam <- list(reference = NULL,
-                         geneFile = NULL,
-                         geneFileFormat = "refFlat",
-                         codonFile = system.file("tabanno/codon.txt", package = "seqminer"),
-                         priorityFile = system.file("tabanno/priority.txt", package = "seqminer"),
-                         upstreamRange = 50,
-                         downstreamRange = 50,
-                         spliceIntoExon = 3,
-                         spliceIntoIntron = 8)
-    ret <- defaultParam
-    for (i in seq_along(param)) {
-        if (names(param)[i] %in% names(ret)) {
-            ret[[names(param)[i]]] <- param[[i]]
-        }
+  ## default parameters
+  defaultParam <- list(reference = NULL,
+                       geneFile = NULL,
+                       geneFileFormat = "refFlat",
+                       codonFile = system.file("tabanno/codon.txt", package = "seqminer"),
+                       priorityFile = system.file("tabanno/priority.txt", package = "seqminer"),
+                       upstreamRange = 50,
+                       downstreamRange = 50,
+                       spliceIntoExon = 3,
+                       spliceIntoIntron = 8,
+                       bed = NULL,
+                       genomeScore = NULL,
+                       tabix = NULL,
+                       indexOutput = FALSE,
+                       inputFormat = "vcf",
+                       checkReference = TRUE)
+  ret <- defaultParam
+  for (i in seq_along(param)) {
+    if (names(param)[i] %in% names(ret)) {
+      ret[[names(param)[i]]] <- param[[i]]
     }
-    ## print(ret)
-    ret
+  }
+  ## print(ret)
+  ret
 }
 
 #' Annotate a test variant
@@ -634,23 +718,23 @@ makeAnnotationParameter <- function(param = NULL) {
 #' print(param)
 #' annotateGene(param, c("1", "1"), c(3, 5) , c("A", "C"), c("G", "C"))
 annotateGene <- function(param, chrom, position, ref, alt) {
-    param <- makeAnnotationParameter(param)
-    res <- validateAnnotationParameter(param)
-    if (!res[[1]])  {
-        cat(res[[2]])
-        stop("Stop due to critical error")
-    }
-    stopifnot(length(chrom) > 0)
-    stopifnot(length(chrom) == length(position))
-    stopifnot(length(chrom) == length(ref))
-    stopifnot(length(chrom) == length(alt))
+  param <- makeAnnotationParameter(param)
+  res <- validateAnnotationParameter(param)
+  if (!res[[1]])  {
+    cat(res[[2]])
+    stop("Stop due to critical error")
+  }
+  stopifnot(length(chrom) > 0)
+  stopifnot(length(chrom) == length(position))
+  stopifnot(length(chrom) == length(ref))
+  stopifnot(length(chrom) == length(alt))
 
-    storage.mode(chrom) <- "character"
-    storage.mode(position) <- "integer"
-    storage.mode(ref) <- "character"
-    storage.mode(alt) <- "character"
+  storage.mode(chrom) <- "character"
+  storage.mode(position) <- "integer"
+  storage.mode(ref) <- "character"
+  storage.mode(alt) <- "character"
 
-    .Call("annotateGene", param, chrom, position, ref, alt, PACKAGE="seqminer")
+  .Call("annotateGene", param, chrom, position, ref, alt, PACKAGE="seqminer")
 }
 
 #' Annotate a test variant
@@ -661,18 +745,18 @@ annotateGene <- function(param, chrom, position, ref, alt) {
 #' @return based extracted from the reference genome
 #' @export
 getRefBase <- function(reference, chrom, position, len = NULL) {
-    if (!file.exists(reference)) {
-        stop("Reference file does not exists")
-    }
-    if (is.null(len)) {
-        len <- rep(1, length(position))
-    }
-    storage.mode(reference) <- "character"
-    storage.mode(chrom) <- "character"
-    storage.mode(position) <- "integer"
-    storage.mode(len) <- "integer"
+  if (!file.exists(reference)) {
+    stop("Reference file does not exists")
+  }
+  if (is.null(len)) {
+    len <- rep(1, length(position))
+  }
+  storage.mode(reference) <- "character"
+  storage.mode(chrom) <- "character"
+  storage.mode(position) <- "integer"
+  storage.mode(len) <- "integer"
 
-    .Call("getRefBase", reference, chrom, position, len, PACKAGE="seqminer")
+  .Call("getRefBase", reference, chrom, position, len, PACKAGE="seqminer")
 }
 
 #' Annotate a VCF file
@@ -689,17 +773,19 @@ getRefBase <- function(reference, chrom, position, len = NULL) {
 #' outVcf <- paste0(getwd(), "/", "out.vcf")
 #' annotateVcf (inVcf, outVcf, param)
 annotateVcf <- function(inVcf, outVcf, params) {
-    params$inputFormat = "vcf"
-    param <- makeAnnotationParameter(param)
-    res <- validateAnnotationParameter(param)
-    if (!res[[1]])  {
-        cat(res[[2]])
-        stop("Stop due to critical error")
-    }
+  params$inputFormat = "vcf"
+  param <- makeAnnotationParameter(param)
+  res <- validateAnnotationParameter(param)
+  if (!res[[1]])  {
+    cat(res[[2]])
+    stop("Stop due to critical error")
+  }
 
-    storage.mode(inVcf) <- "character"
-    storage.mode(outVcf) <- "character"
-    .Call("anno", inVcf, outVcf, params)
+  verifyFilename(inVcf, outVcf)
+  
+  storage.mode(inVcf) <- "character"
+  storage.mode(outVcf) <- "character"
+  .Call("anno", inVcf, outVcf, params)
 }
 
 #' Annotate a plain text file
@@ -717,16 +803,18 @@ annotateVcf <- function(inVcf, outVcf, params) {
 #' outFile <- paste0(getwd(), "/", "out.annotated.txt")
 #' annotatePlain(inFile, outFile, param)
 annotatePlain <- function(inFile, outFile, params) {
-    params$inputFormat = "plain"
-    param <- makeAnnotationParameter(param)
-    res <- validateAnnotationParameter(param)
-    if (!res[[1]])  {
-        cat(res[[2]])
-        stop("Stop due to critical error")
-    }
-    storage.mode(inFile) <- "character"
-    storage.mode(outFile) <- "character"
-    .Call("anno", inFile, outFile, params)
+  params$inputFormat = "plain"
+  param <- makeAnnotationParameter(param)
+  res <- validateAnnotationParameter(param)
+  if (!res[[1]])  {
+    cat(res[[2]])
+    stop("Stop due to critical error")
+  }
+  verifyFilename(inFile, outFile)
+  
+  storage.mode(inFile) <- "character"
+  storage.mode(outFile) <- "character"
+  .Call("anno", inFile, outFile, params)
 }
 
 #' Test whether a vector of positions are inside given ranges
@@ -739,26 +827,26 @@ annotatePlain <- function(inFile, outFile, params) {
 #' ranges <- "1:1-3,1:2-4,1:5-10"
 #' isInRange(positions, ranges)
 isInRange <- function(positions, rangeList) {
-    storage.mode(positions) <- "character"
-    storage.mode(rangeList) <- "character"
-    .Call("isInRange", positions, rangeList)
+  storage.mode(positions) <- "character"
+  storage.mode(rangeList) <- "character"
+  .Call("isInRange", positions, rangeList)
 }
 
 #' Extract pair of positions by ranges
 #' @param covData a covariance matrix with positions as dimnames
 #' @param rangeList1 character specify a range
 #' @param rangeList2 character specify a range
-#' @return a covariance matrix 
+#' @return a covariance matrix
 #' covFileName = system.file("rvtests/rvtest.MetaCov.assoc.gz", package = "seqminer")
 #' cfh <- rvmeta.readCovByRange(covFileName, "1:196621007-196716634")
 #' rangeList1 <- "1:196621007-196700000"
 #' rangeList2 <- "1:196700000-196716634"
 #' getCovPair(cfh, rangeList1, rangeList2)
 getCovPair <- function(covData, rangeList1, rangeList2) {
-    pos <- dimnames(covData)[1][[1]]
-    idx1 <- isInRange(pos, rangeList1)
-    idx2 <- isInRange(pos, rangeList2)
-    ret <- covData[idx1, idx2]
-    dimnames(ret) <- list(pos[idx1], pos[idx2])
-    ret
+  pos <- dimnames(covData)[1][[1]]
+  idx1 <- isInRange(pos, rangeList1)
+  idx2 <- isInRange(pos, rangeList2)
+  ret <- covData[idx1, idx2]
+  dimnames(ret) <- list(pos[idx1], pos[idx2])
+  ret
 }
